<#
.SYNOPSIS
    .
.DESCRIPTION
    .
.PARAMETER ResourceGroup
    Please select the ResourceGroup to target.
.PARAMETER TargetVM
    Used if adding a disk to a single VM
.PARAMETER VMPrefix
    Used if adding a disk to a set of VM's in a ResourceGroup starting with the name specified in "VMPrefix" (eg VMnode*)
.PARAMETER MaxDisks
    Used when adding multiple disks to VM's - All VM's in scope will get equal number of disks
.PARAMETER SizeinGB
    Size of disks to be added
.PARAMETER Cache
    Specify none, ReadOnly, ReadWrite
.PARAMETER Log
    If -Log $true, a logfile will be generated in the current directory
.PARAMETER Login
    Specify requirement for login (future add)
.EXAMPLE
    C:\PS>DiskAdd-toVM.ps1  -ResourceGroup 'MyResourceGroup' -VMTarget "VMName" -SizeInGB 1023 -Cache none
    <Will add a single 1TB disk to VMName VM>

    C:\PS>DiskAdd-toVM.ps1  -ResourceGroup 'MyResourceGroup' -SizeInGB 1023 -Cache none
    <Will add a single 1TB disk to ALL VM"s in Resource Group MyResourceGroup>

    C:\PS>DiskAdd-toVM.ps1  -ResourceGroup 'MyResourceGroup' -VMPrefix 'MyClusterNode'-SizeInGB 1023 -Cache none
    <Will add a single 1TB disk to ALL VM"s where the name starts with 'MyClusterNode' in Resource Group MyResourceGroup>

    C:\PS>DiskAdd-toVM.ps1  -ResourceGroup 'MyResourceGroup' -VMPrefix 'MyClusterNode'-SizeInGB 1023 -Cache none -MaxDisks 10
    <Will ensure all VM"s where the name starts with 'MyClusterNode' in Resource Group MyResourceGroup will have at least 10 disks, 
    new disks added will be 1Tb in size>

.NOTES
    Author: Roelf Zomerman
    Date:   April 14, 2017
#>

 [CmdletBinding(DefaultParameterSetName="ResourceGroup")]

Param(
	[Parameter(
		Mandatory=$true,
		HelpMessage="Type the ResourceGroup"
		)]
	    [ValidateNotNullOrEmpty()]
       
	[Alias('Please provide the ResourceGroup')]	
    	$ResourceGroup, #ResourceGroupToTarget
    [Parameter(
		Mandatory=$false
		)]
    	$TargetVM,
    [Parameter(
		Mandatory=$false
		)]
    	$VMPrefix,
	[Parameter(
		Mandatory=$false
		)]
	    $Login,
    [Parameter(
		Mandatory=$false
		)]
	    $MaxDisks,
    [Parameter(
		Mandatory=$true
		)]
	    $SizeinGB,
    [Parameter(
		Mandatory=$true
		)]
	    $Cache,
    [Parameter(
		Mandatory=$false
		)]
	    $Log
)

Function ActivateDebug(){
			Add-Content -Path $LogfileActivated -Value "***************************************************************************************************"
			Add-Content -Path $LogfileActivated -Value "Started processing at [$([DateTime]::Now)]."
			Add-Content -Path $LogfileActivated -Value "***************************************************************************************************"
			Add-Content -Path $LogfileActivated -Value ""
			Write-Host "Debug Enabled writing to logfile: " $LogfileActivated
}

Function WriteDebug{
    [CmdletBinding()]
    Param ([Parameter(Mandatory=$true)][string]$LineValue)
        Write-host $LineValue
        #If $Log, also write to file.. else only console output
        If ($log) {
        	Process{
    		Add-Content -Path $LogfileActivated -Value $LineValue
            }
        }
}

#/GET THE VM's'
Function GetOperationsTarget ($ResourceGroupName){
    If ($VMprefix) {
        #a prefix was specified - all VM's in a ResGroup with prefix selected
        $searchString=($VMprefix + '*')
        $coll=Get-AzureRmVM -ResourceGroupName $ResourceGroupName | Where-Object { $_.Name -like $searchString}  
        WriteDebug "Targetted VM's"
        foreach ($entry in $coll) {
            $value= (" -> " + $entry.name)
            WriteDebug $value    
        }
        WriteDebug "The VM's above are selected to have an addition disk added"
        $x = Read-Host 'Press any key to exit or press P to continue'
        If ($x.toUpper() -ne "P") {
            WriteDebug "SAFE QUIT" -ForegroundColor Green
            exit
        }
    }
    #no single target or prefix specified - providing warning of all VM's in Res Group
    If (!($TargetVM) -and (!($VMPrefix))) {
    WriteDebug ("Selected ResourceGroup: " + $ResourceGroupName)
    $coll=Get-AzureRmVM -ResourceGroupName $ResourceGroupName
            foreach ($entry in $coll) {
            $value= (" -> " + $entry.name)
            WriteDebug $value
        }
        Write-host "The VM's above are selected to have an addition disk added"
        $x = Read-Host 'Press any key to exit or press P to continue'
        If ($x.toUpper() -ne "P") {
            WriteDebug "SAFE QUIT" -ForegroundColor Green
            exit
        }
    }
    
    return $coll
    
}

Function AddDiskToVM ($VMCollection) {
    Foreach ($VM in $VMCollection) {
        #/THIS WILL ADD A NEW DISK TO A VM - including the disknumer etc..
            $vmconfig=get-azurermvm -ResourceGroupName $vm.resourceGroupName -ResourceName $vm.name
            $myUri=$vmconfig.StorageProfile.OsDisk.Vhd.Uri
            If (!($myUri)){ 
                WriteDebug "Managed Disks or no error in getting disk profile - NEXT VM"
                continue
            }
            $count=$myUri.LastIndexOf("/")
            $sourceUri=($myUri.Substring(0,$count) + "/")
        #/DEFINED THE TARGET STORAGE ACCOUNT FOR NEW DISK
        # DISK TO BE ADDED = $targetUri + diskname.vhd
        #/
        #/ Count the number of disks now attached to the VM
        $disknumber=$vmconfig.DataDiskNames.count

    #Add DiskCounter
    #If no MaxDisks was specified, maxDisks = Disknumber +1 
    If (!($MaxDisks)) {$MaxDisks = $disknumber +1}
        #If MaxDisks was specified, we need to add the number of disks to match maxdisks to each VM
        If ($MaxDisks -gt $disknumber) {
            Do {
                WriteDebug ("Need to add " + ($MaxDisks - $DiskNumber) + " disks")
                #/Construction of the DataDiskNamess - helixclouddn0datadiskXStorageAccountName.vhd
                $Diskname=($vm.name + "datadisk" + $disknumber)
                $targetUri=($sourceUri + $diskName + ".vhd")
                Do {
                    $disknumber++
                    $Diskname=($vm.name + "datadisk" + $disknumber)
                    $targetUri=($sourceUri + $diskName + ".vhd")
                }
                Until ($vmconfig -notcontains $targetUri)

            #/Get highest LUN number
                $maxLun = ($vm.StorageProfile.DataDisks | measure-object -Property LUN -maximum).maximum
                $lun=$maxLun+1
            #/
                WriteDebug ("Will be adding " + $diskName )
                WriteDebug (" with URI " + $targetUri )
                WriteDebug (" to VM " + $vm.name + " in ResGrp " + $vm.resourceGroupName )
                #WriteDebug ($vm.getType())
                Add-AzureRmVMDataDisk -VM $vm -Name $diskName -VhdUri $targetUri -Caching $Cache -DiskSizeinGB $SizeinGB -CreateOption Empty -lun $lun
                Update-AzureRmVM -ResourceGroupName $vm.resourceGroupName -VM $vm
                
            }   
            While ($MaxDisks -ne $disknumber) 
        }
        #Schedule / install extension - to push auto format
        #
        #
        # TODO
        #

        $vm=$false
    }
}


If ($Log){
			$date=(Get-Date).ToString("d-M-y-h.m.s")
			$logname = ("AddDisk-" + $date + ".log")
			New-Item -Path $pwd.path -Value $LogName -ItemType File
			$LogfileActivated=$pwd.path + "\" + $LogName
	        ActivateDebug
} #Activating DEBUG MODE

#Activating script, validating Azure CMD'lets
Try {
	Import-Module Azure.Storage
	}
	catch {
	Write-Host 'Modules NOT LOADED - EXITING'
	Exit
}


#ACTUAL LOGIC GOES HERE
#IF $TargetVM was specified, we only need to add 1 disk to a VM.. 
If ($TargetVM) {
    [array]$VMCollection=Get-AzureRmVm -ResourceGroupName $ResourceGroup -Name $TargetVM
}
Else {
    #no TargetVM, as such, we need to select operations target based on ResourceGroup and possibly $VMPrefix
     $VMCollection=GetOperationsTarget $ResourceGroup
}


#COUNTING LOGIC for number of DISKS

AddDiskToVM $VMCollection

#Get-OperationsTarget
#VM or resourceGroup
#NumberOfDisksToAdd/MaxDisks
#AddDisks (until number reached)